@@ -16,7 +16,7 @@ test_that("install_maven() detects existing installation", {
 
 test_that("install_mleap() works", {
   skip_on_cran()
-  if (mleap:::mleap_found()) skip("MLeap already installed")
+  if (mleap:::mleap_found(.globals$default_mleap_version)) skip("MLeap already installed")
   expect_message(
     install_mleap(dir = mleap_dir),
     paste0("MLeap Runtime version ",
